@@ -75,6 +75,15 @@ class ContinuousDomain(Domain):
         self.domain_args = args
         self.domain_kwargs = kwargs
 
+    @property
+    def bounds(self):
+        loc = self.domain_kwargs['loc'] if 'loc' in self.domain_kwargs \
+                else 0
+        scl = self.domain_kwargs['scale'] if 'scale' in self.domain_kwargs \
+                else 1
+        return self.domain.interval(0.999, *self.domain_args,
+                                    loc=loc, scale=scl)
+
     @property
     def complexity(self):
         if self._complexity is None:
@@ -107,6 +116,10 @@ class ContinuousDomain(Domain):
         domain = cls(obj['name'], obj['domain'], *obj['domain_args'],
                      callback=callback, seed=random_state,
                      **obj['domain_kwargs'])
+        
+        domain.id = obj['id']
+        domain.current = obj['current']
+        
         return domain
 
     def generate(self):
