@@ -12,6 +12,7 @@ import itertools
 from multiprocessing.pool import ThreadPool
 import operator
 import os
+from typing import Iterable
 import warnings
 
 import numpy as np
@@ -46,6 +47,7 @@ class SearchSpace(object, metaclass=SearchSpaceMeta):
         self.id = next(self.__class__._counter)
         self.exp_key = exp_key
         self.trials = []
+        self.ready = True
 
         self._complexity = None
         self._uncertainty = None
@@ -61,9 +63,6 @@ class SearchSpace(object, metaclass=SearchSpaceMeta):
                         d1.domain = d2 
         self.domains.sort()
 
-
-        self.done = False
-
     def __call__(self, method=None, to_dict=False):
         """Generate a new trial for this search space.
 
@@ -113,6 +112,10 @@ class SearchSpace(object, metaclass=SearchSpaceMeta):
                 1.0)
         return self._complexity
 
+    def done(self, max_evals):
+        complete = sum([1 if t.status == TrialStatus.DONE else 0 for t in self.trials])
+        return complete >= max_evals
+
     @classmethod
     def from_json(cls, obj):
         """Recreate a SearchSpace from its JSON representation.
@@ -191,7 +194,10 @@ class SearchSpace(object, metaclass=SearchSpaceMeta):
             for i, result in enumerate(completed):
                 vec = [float(self.domains[j].map_to_domain(result.hyperparameters[j]))
                     for j in range(len(self.domains))]
-                vec.append(result.objective)
+                if isinstance(result.objective, Iterable):
+                    vec.extend(result.objective)
+                else:
+                    vec.append(result.objective)
                 out[i] += np.asarray(vec)
         else:
             out = None
@@ -230,14 +236,12 @@ class SearchSpace(object, metaclass=SearchSpaceMeta):
         df = pd.DataFrame.from_dict(df_dict)
         return df
 
-            
-
     def to_json(self, simplify=False):
         """Convert this search space to a JSON-compatible representation."""
         jsonified = {
             'exp_key': self.exp_key,
             'complexity': self._complexity,
-            'uncertainty': self._uncertainty
+            'uncertainty': self._uncertainty,
         }
 
         if not simplify:
@@ -325,11 +329,11 @@ class PopulationSearchSpace(SearchSpace):
     
     """
 
-    def __init__(self, domains, population_size=50, exp_key=''):
+    def __init__(self, domains, exp_key=''):
         super().__init__(domains, exp_key=exp_key)
-        self.population_size = population_size
-        self.population = []
+        self.population = None
         self.best = None
+        self.generations = 0
 
     def __call__(self, method=None, to_dict=False):
         """Generate a new trial for this search space.
@@ -347,9 +351,37 @@ class PopulationSearchSpace(SearchSpace):
             nested dictionary of hyperparameter values matching the structure
             of the original specification.
         """
-        if method is None:
-            method = random_search
-        self.population = method(self)
-        trials = [Trial(self, hyperparameters=h) for h in self.population]
-        self.trials.extend(trials)
-        return [t.parameter_dict for t in trials] if to_dict else trials
\ No newline at end of file
+        print(self.population)
+        self.ready = self.population is None or all([t.objective is not None for t in self.population])
+
+        if self.ready:
+            if method is None:
+                method = random_search
+            population = method(self)
+            self.population = [Trial(self, hyperparameters=h) for h in population]
+            self.trials.extend(self.population)
+            self.generations += 1
+            return [t.parameter_dict for t in self.poulation] if to_dict else self.population
+        else:
+            return []
+
+    def done(self, max_evals):
+        return self.generations >= max_evals
+
+    def population_to_array(self):
+        if self.population is not None:
+            out = np.zeros((len(self.population), len(self.domains) + 1),
+                           dtype=np.float32)
+
+            for i, result in enumerate(self.trials[-len(self.population):]):
+                vec = [float(self.domains[j].map_to_domain(result.hyperparameters[j]))
+                       for j in range(len(self.domains))]
+                if isinstance(result.objective, Iterable):
+                    vec.extend(result.objective)
+                else:
+                    vec.append(result.objective)
+                out[i] += np.asarray(vec)
+
+            return out
+        else:
+            return None
